@@ -51,11 +51,15 @@
 #include <libgnome/gnome-macros.h>
 
 #include "gnome-calc.h"
+#include "sr.h"
+
+#undef GNOME_CALC_DEBUG
 
 typedef void (*sighandler_t)(int);
 
 struct _GnomeCalcPrivate {
 	GtkWidget *text_display;
+	GtkWidget *store_display;
 
 	GtkWidget *invert_button;
 	GtkWidget *drg_button;
@@ -107,7 +111,7 @@ static void gnome_calc_finalize	(GObject		*object);
 typedef struct _CalculatorButton CalculatorButton;
 struct _CalculatorButton {
 	char *name;
-	GtkSignalFunc signal_func;
+	GCallback signal_func;
 	gpointer data;
 	gpointer invdata;
 	gboolean convert_to_rad;
@@ -129,6 +133,8 @@ enum {
 
 static gint gnome_calc_signals[LAST_SIGNAL];
 
+static gboolean cmd_saved = FALSE;
+
 GNOME_CLASS_BOILERPLATE (GnomeCalc, gnome_calc,
 			 GtkVBox, GTK_TYPE_VBOX)
 
@@ -157,7 +163,7 @@ gnome_calc_class_init (GnomeCalcClass *class)
 	class->result_changed = NULL;
 }
 
-#if 0 /*only used for debugging*/
+#ifdef GNOME_CALC_DEBUG 
 static void
 dump_stack(GnomeCalc *gc)
 {
@@ -211,12 +217,31 @@ stack_pop(GList **stack)
 static void
 set_display (GnomeCalc *gc)
 {
-	GtkWidget *label = gc->_priv->text_display;
-	gchar *string;
+        GtkWidget *string_text = gc->_priv->text_display;
+        GtkWidget *store_text = gc->_priv->store_display;
+        GtkTextBuffer *buffer;
+        GtkTextIter start, end;
+
+        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (string_text));
+        gtk_text_buffer_get_bounds (buffer, &start, &end);
+        gtk_text_buffer_delete (buffer, &start, &end);
+        gtk_text_buffer_insert_with_tags_by_name (buffer,
+                                                  &end,
+                                                  gc->_priv->result_string,
+                                                  -1,
+                                                  "x-large",
+                                                  NULL);
+        
+	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (store_text));
+        gtk_text_buffer_get_bounds (buffer, &start, &end);
+        gtk_text_buffer_delete (buffer, &start, &end);
+        gtk_text_buffer_insert_with_tags_by_name (buffer,
+                                                  &end,
+                                                  (gc->_priv->memory != 0.0) ? "m" : " ",
+                                                  -1,
+                                                  "x-large",
+                                                  NULL);
 
-	string = g_strconcat ("<span foreground=\"white\"  font_desc=\"Helvetica 24\">", gc->_priv->result_string, "</span>", NULL);
-	gtk_label_set_markup (GTK_LABEL (label), string);
-	g_free (string);
 }
 
 static void
@@ -494,6 +519,9 @@ convert_num(gdouble num, GnomeCalcMode from, GnomeCalcMode to)
 static void
 no_func(GtkWidget *w, gpointer data)
 {
+	GList *list;
+	static CalculatorStack saved_func, saved_num;
+	CalculatorStack *stack1, *stack2, *add_func, *add_num;
 	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
 
 	g_return_if_fail(gc!=NULL);
@@ -515,6 +543,50 @@ no_func(GtkWidget *w, gpointer data)
 		gc->_priv->error = FALSE;
 		return;
 	}
+
+	/* = after <math_func>=  eg: 3+== */
+	if (cmd_saved == TRUE) {
+		add_func = g_new(CalculatorStack, 1);
+		add_func->type = saved_func.type;
+		add_func->d = saved_func.d;
+		gc->_priv->stack = g_list_prepend(gc->_priv->stack, add_func);
+
+		add_num = g_new(CalculatorStack, 1);
+		add_num->type = saved_num.type;
+		add_num->d = saved_num.d;
+		gc->_priv->stack = g_list_prepend(gc->_priv->stack, add_num);
+		gc->_priv->add_digit = FALSE;
+
+		reduce_stack(gc);
+	} else {
+		if(gc->_priv->stack == NULL)
+			return;
+		stack1 = gc->_priv->stack->data;
+
+		list=g_list_next(gc->_priv->stack);
+		if(list) {
+			stack2 = list->data;
+
+			if( stack1->type == CALCULATOR_FUNCTION && 
+		    		stack2->type == CALCULATOR_NUMBER) {
+				saved_func.type = CALCULATOR_FUNCTION;
+				saved_func.d = stack1->d;
+
+				saved_num.type = CALCULATOR_NUMBER;
+				saved_num.d = stack2->d;
+
+				add_num = g_new(CalculatorStack,1);
+				add_num->type = saved_num.type;
+				add_num->d = saved_num.d;
+				gc->_priv->stack = g_list_prepend(gc->_priv->stack, add_num);
+				gc->_priv->add_digit = FALSE;
+
+				reduce_stack(gc);
+				cmd_saved = TRUE;
+			}
+		}
+	}
+
 	set_result(gc);
 
 	unselect_invert(gc);
@@ -532,6 +604,8 @@ simple_func(GtkWidget *w, gpointer data)
 	g_return_if_fail(func!=NULL);
 	g_return_if_fail(gc!=NULL);
 
+	cmd_saved = FALSE;
+
 	if(gc->_priv->error)
 		return;
 
@@ -543,6 +617,12 @@ simple_func(GtkWidget *w, gpointer data)
 	}
 
 	stack = gc->_priv->stack->data;
+
+	if(stack->type==CALCULATOR_FUNCTION) {
+		stack_pop(&gc->_priv->stack);
+		stack = gc->_priv->stack->data;
+	}
+
 	if(stack->type!=CALCULATOR_NUMBER) {
 		unselect_invert(gc);
 		return;
@@ -596,6 +676,8 @@ math_func(GtkWidget *w, gpointer data)
 	g_return_if_fail(func!=NULL);
 	g_return_if_fail(gc!=NULL);
 
+	cmd_saved = FALSE;
+
 	if(gc->_priv->error)
 		return;
 
@@ -611,6 +693,12 @@ math_func(GtkWidget *w, gpointer data)
 	set_result(gc);
 
 	stack = gc->_priv->stack->data;
+
+	if(stack->type==CALCULATOR_FUNCTION) {
+		stack_pop(&gc->_priv->stack);
+		stack = gc->_priv->stack->data;
+	}
+
 	if(stack->type!=CALCULATOR_NUMBER) {
 		unselect_invert(gc);
 		return;
@@ -639,6 +727,8 @@ reset_calc(GtkWidget *w, gpointer data)
 
 	g_return_if_fail(gc!=NULL);
 
+	cmd_saved = FALSE;
+
 	while(gc->_priv->stack)
 		stack_pop(&gc->_priv->stack);
 
@@ -652,7 +742,6 @@ reset_calc(GtkWidget *w, gpointer data)
 	gc->_priv->add_digit = TRUE;
 	push_input(gc);
 	set_result(gc);
-
 	unselect_invert(gc);
 	setup_drg_label(gc);
 }
@@ -664,6 +753,8 @@ clear_calc(GtkWidget *w, gpointer data)
 
 	g_return_if_fail(gc!=NULL);
 
+	cmd_saved = FALSE;
+
 	/* if in add digit mode, just clear the number, otherwise clear
 	 * state as well */
 	if(!gc->_priv->add_digit) {
@@ -710,6 +801,8 @@ add_digit (GtkWidget *w, gpointer data)
 	CalculatorButton *but = data;
 	gchar *digit = but->name;
 
+	cmd_saved = FALSE;
+
 	if(gc->_priv->error)
 		return;
 
@@ -738,7 +831,7 @@ add_digit (GtkWidget *w, gpointer data)
 		else if(strlen(gc->_priv->result_string)>9)
 			return;
 		else if(gc->_priv->result_string[0]=='\0')
-			strcpy(gc->_priv->result_string," 1");
+			strcpy(gc->_priv->result_string," 0");
 	} else if(digit[0]=='.') {
 		if(strchr(gc->_priv->result_string,'.'))
 			return;
@@ -779,12 +872,14 @@ negate_val(GtkWidget *w, gpointer data)
 
 	g_return_if_fail(gc!=NULL);
 
+	cmd_saved = FALSE;
+
 	if(gc->_priv->error)
 		return;
 
 	unselect_invert(gc);
 
-	if(!gc->_priv->add_digit) {
+	if(!gc->_priv->add_digit && gc->_priv->result != 0) {
 		simple_func(w,data);
 		return;
 	}
@@ -797,9 +892,9 @@ negate_val(GtkWidget *w, gpointer data)
 			*p='-';
 	} else {
 		if(gc->_priv->result_string[0]=='-' || 
-		   gc->_priv->result == 0)
+		   gc->_priv->result == 0) 
 			gc->_priv->result_string[0]=' ';
-		else
+		else 
 			gc->_priv->result_string[0]='-';
 	}
 
@@ -900,7 +995,7 @@ static void
 store_m(GtkWidget *w, gpointer data)
 {
 	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
-	
+
 	g_return_if_fail(gc!=NULL);
 
 	if(gc->_priv->error)
@@ -1030,13 +1125,23 @@ set_pi(GtkWidget *w, gpointer data)
 	unselect_invert(gc);
 }
 
+static gboolean
+maybe_run_slide_rule (GtkWidget *w, GdkEvent *event, gpointer data)
+{
+	if (event->type == GDK_3BUTTON_PRESS) {
+		run_slide_rule ();
+		return TRUE;
+	}
+	return FALSE;
+}
+
 static void
-set_e(GtkWidget *w, gpointer data)
+set_e (GtkWidget *w, gpointer data)
 {
 	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
 
 	g_return_if_fail(gc!=NULL);
-
+	
 	if(gc->_priv->error)
 		return;
 
@@ -1052,6 +1157,8 @@ add_parenth(GtkWidget *w, gpointer data)
 
 	g_return_if_fail(gc!=NULL);
 
+	cmd_saved = FALSE;
+
 	if(gc->_priv->error)
 		return;
 
@@ -1075,6 +1182,8 @@ sub_parenth(GtkWidget *w, gpointer data)
 	GnomeCalc *gc = g_object_get_data(G_OBJECT(w), "set_data");
 	g_return_if_fail(gc!=NULL);
 
+	cmd_saved = FALSE;
+
 	if(gc->_priv->error)
 		return;
 
@@ -1141,7 +1250,7 @@ tan_helper (double value)
 static const CalculatorButton buttons[8][5] = {
 	{
 		{N_("1/x"),  (GtkSignalFunc)simple_func, c_inv,  NULL,   FALSE, {0}, N_("Inverse"), NULL },
-		{N_("x^2"),  (GtkSignalFunc)simple_func, c_pow2, sqrt,   FALSE, {0}, N_("Square"), NULL },
+		{N_("x<sup>2</sup>"),  (GtkSignalFunc)simple_func, c_pow2, sqrt,   FALSE, {0}, N_("Square"), NULL },
 		{N_("SQRT"), (GtkSignalFunc)simple_func, sqrt,   c_pow2, FALSE, {'r','R',0}, N_("Square root"), NULL },
 		{N_("CE/C"), (GtkSignalFunc)clear_calc,  NULL,   NULL,   FALSE, {GDK_Clear,GDK_Delete,0}, N_("Clear"), NULL },
 		{N_("AC"),   (GtkSignalFunc)reset_calc,  NULL,   NULL,   FALSE, {'a','A', GDK_Escape, 0}, N_("Reset"), NULL }
@@ -1157,7 +1266,7 @@ static const CalculatorButton buttons[8][5] = {
 		{N_("EE"),   (GtkSignalFunc)add_digit,   "e+",   NULL,   FALSE, {0}, N_("Add digit"), NULL },
 		{N_("log"),  (GtkSignalFunc)simple_func, log10,  c_pow10,FALSE, {0}, N_("Base 10 Logarithm"), NULL },
 		{N_("ln"),   (GtkSignalFunc)simple_func, log,    c_powe, FALSE, {'l','L',0}, N_("Natural Logarithm"), NULL },
-		{N_("x^y"),  (GtkSignalFunc)math_func,   pow,    NULL,   FALSE, {'^',0}, N_("Power"), NULL }
+		{N_("x<sup>y</sup>"),  (GtkSignalFunc)math_func,   pow,    NULL,   FALSE, {'^',0}, N_("Power"), NULL }
 	},{
 		{N_("PI"),   (GtkSignalFunc)set_pi,      NULL,   NULL,   FALSE, {'p','P',0}, N_("PI"), NULL },
 		{N_("x!"),   (GtkSignalFunc)simple_func, c_fact, NULL,   FALSE, {'!',0}, N_("Factorial"), NULL },
@@ -1195,23 +1304,34 @@ static void
 create_button(GnomeCalc *gc, GtkWidget *table, int x, int y)
 {
 	const CalculatorButton *but = &buttons[y][x];
-	GtkWidget *w;
+	GtkWidget *w, *l;
 	int i;
 
 	if(!but->name)
 		return;
 
+	l = gtk_label_new (_(but->name));
+	gtk_widget_show (l);
+	gtk_label_set_use_markup (GTK_LABEL (l), TRUE);
+
 	if (strcmp (but->name, "INV") == 0) {
-		w = gtk_toggle_button_new_with_label(_("INV"));
+		w = gtk_toggle_button_new ();
+		gtk_container_add (GTK_CONTAINER (w), l);
 		gc->_priv->invert_button = w;
-		gtk_signal_connect (GTK_OBJECT (w), "toggled",
-				    GTK_SIGNAL_FUNC(invert_toggle), gc);
+		g_signal_connect (G_OBJECT (w), "toggled",
+				  G_CALLBACK (invert_toggle), gc);
 	} else {
-		w = gtk_button_new_with_label(_(but->name));
-		gtk_signal_connect(GTK_OBJECT(w), "clicked",
-				   but->signal_func,
-				   (gpointer) but);
+		w = gtk_button_new ();
+		gtk_container_add (GTK_CONTAINER (w), l);
+		g_signal_connect (G_OBJECT(w), "clicked",
+				  but->signal_func,
+				  (gpointer) but);
 	}
+
+	if (strcmp (but->name, "e") == 0)
+		g_signal_connect (G_OBJECT (w), "button_press_event",
+				  G_CALLBACK (maybe_run_slide_rule),
+				  (gpointer) but);
 	
 	for(i=0;but->keys[i]!=0;i++) {
 		gtk_widget_add_accelerator(w, "clicked",
@@ -1243,36 +1363,53 @@ create_button(GnomeCalc *gc, GtkWidget *table, int x, int y)
 			 GTK_FILL | GTK_EXPAND | GTK_SHRINK, 2, 2);
 
 	/* if this is the DRG button, remember it's pointer */
-	if(but->signal_func == GTK_SIGNAL_FUNC(drg_toggle))
+	if(but->signal_func == G_CALLBACK (drg_toggle))
 		gc->_priv->drg_button = w;
 }
 
 static void
 gnome_calc_instance_init (GnomeCalc *gc)
 {
-	GtkWidget *hbox, *event;
-	gint x,y;
+	GtkWidget *hbox;
 	GtkWidget *table;
-	GdkColor color;
+	GtkTextBuffer *buffer;
+	gint x,y;
 	
 	gc->_priv = g_new0(GnomeCalcPrivate, 1);
 	
-	event = gtk_event_box_new ();
-	gtk_widget_show (event);
-	gdk_color_parse ("#000000", &color);
-	gtk_widget_modify_bg (event, GTK_STATE_NORMAL, &color); 
-	gtk_box_pack_start (GTK_BOX (gc), event, FALSE, FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (event), 1);
-	
 	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
 	gtk_widget_show (hbox);
-	gtk_container_add (GTK_CONTAINER (event), hbox);
+	gtk_box_pack_start (GTK_BOX (gc), hbox, FALSE, FALSE, 0);
 	
-	gc->_priv->text_display = gtk_label_new (NULL);
-	gtk_label_set_selectable (GTK_LABEL (gc->_priv->text_display), TRUE);
-	/*gtk_label_set_justify (GTK_LABEL (gc->_priv->text_display), GTK_JUSTIFY_RIGHT);*/
+	gc->_priv->text_display = gtk_text_view_new ();
+	gc->_priv->store_display = gtk_text_view_new ();
+
+	g_object_set (G_OBJECT (gc->_priv->store_display), 
+		      "cursor-visible", FALSE, NULL);
+	
+        gtk_text_view_set_editable (GTK_TEXT_VIEW (gc->_priv->store_display), FALSE);
+        gtk_text_view_set_justification (GTK_TEXT_VIEW (gc->_priv->store_display),
+                                         GTK_JUSTIFY_LEFT);
+        gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (gc->_priv->store_display),
+                                     GTK_WRAP_WORD);
+        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (gc->_priv->store_display));
+        gtk_text_buffer_create_tag (buffer, "x-large",
+                                    "scale", PANGO_SCALE_XX_LARGE, NULL);
+        gtk_box_pack_start (GTK_BOX (hbox), gc->_priv->store_display, TRUE, TRUE, 0);
+
+        gtk_text_view_set_editable (GTK_TEXT_VIEW (gc->_priv->text_display), FALSE);
+        gtk_text_view_set_justification (GTK_TEXT_VIEW (gc->_priv->text_display),
+                                         GTK_JUSTIFY_RIGHT);
+        gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (gc->_priv->text_display),
+                                     GTK_WRAP_WORD);
+        buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (gc->_priv->text_display));
+        gtk_text_buffer_create_tag (buffer, "x-large",
+                                    "scale", PANGO_SCALE_XX_LARGE, NULL);
+        gtk_box_pack_end (GTK_BOX (hbox), gc->_priv->text_display, TRUE, TRUE, 0);
+
 	gtk_widget_show (gc->_priv->text_display);
-	gtk_box_pack_end (GTK_BOX (hbox), gc->_priv->text_display, FALSE, TRUE, 0);	
+	gtk_widget_show (gc->_priv->store_display);
 	
 	gc->_priv->stack = NULL;
 	gc->_priv->result = 0;
@@ -1542,7 +1679,7 @@ gnome_calc_bind_extra_keys (GnomeCalc *gc,
 
 	g_object_set_data (G_OBJECT (widget), "set_data", gc);
 	
-	gtk_signal_connect (GTK_OBJECT (widget), "event",
-			    GTK_SIGNAL_FUNC (event_cb), gc);
+	g_signal_connect (GTK_OBJECT (widget), "event",
+			  G_CALLBACK (event_cb), gc);
 
 }
